@@ -1,8 +1,8 @@
 package org.broadinstitute.barclay.argparser;
 
-import org.apache.commons.lang3.tuple.Pair;
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
+import org.broadinstitute.barclay.utils.Pair;
 import org.broadinstitute.barclay.utils.Utils;
 
 import java.io.PrintStream;
@@ -42,6 +42,7 @@ public class NamedArgumentDefinition extends ArgumentDefinition {
      *                                      {@code CommandLinePluginDescriptor} itself are not contained in a plugin,
      *                                      and will not have a {@code descriptorForControllingPlugin}. can be null.
      */
+    @SuppressWarnings("this-escape")
     public NamedArgumentDefinition(
             final Argument argumentAnnotation,
             final Object containingObject,
@@ -311,7 +312,7 @@ public class NamedArgumentDefinition extends ArgumentDefinition {
                             "A \"null\" value was detected for an option after values for that option were already set. " +
                                     "Clobbering previously set values for this option: %s.", getArgumentAliasDisplayString()));
                 }
-                if (!isOptional()) {
+                if (!argumentAnnotation.optional() && i == preprocessedValues.size() - 1) {
                     throw new CommandLineException(
                             String.format("Non \"null\" value must be provided for '%s'", getArgumentAliasDisplayString()));
                 }
@@ -704,4 +705,4 @@ public class NamedArgumentDefinition extends ArgumentDefinition {
         result = 31 * result + (getDefaultValueAsString() != null ? getDefaultValueAsString().hashCode() : 0);
         return result;
     }
-}
\ No newline at end of file
+}
