@@ -1,12 +1,10 @@
 package org.broadinstitute.barclay.help;
 
-import jdk.javadoc.doclet.DocletEnvironment;
-import org.apache.commons.lang3.tuple.Pair;
-
 import org.apache.logging.log4j.LogManager;
 import org.apache.logging.log4j.Logger;
 import org.broadinstitute.barclay.argparser.*;
 import org.broadinstitute.barclay.help.scanners.JavaLanguageModelScanners;
+import org.broadinstitute.barclay.utils.Pair;
 import org.broadinstitute.barclay.utils.Utils;
 
 import javax.lang.model.element.Element;
