@@ -2,8 +2,6 @@ package org.broadinstitute.barclay.help;
 
 import com.google.gson.Gson;
 import com.google.gson.GsonBuilder;
-import com.sun.javadoc.ClassDoc;
-import com.sun.javadoc.RootDoc;
 
 import freemarker.cache.TemplateLoader;
 import freemarker.cache.FileTemplateLoader;
@@ -14,11 +12,16 @@ import freemarker.template.DefaultObjectWrapper;
 import freemarker.template.Template;
 import freemarker.template.TemplateException;
 
-import org.apache.logging.log4j.LogManager;
-import org.apache.logging.log4j.Logger;
+import jdk.javadoc.doclet.DocletEnvironment;
+import jdk.javadoc.doclet.Doclet;
+import jdk.javadoc.doclet.Reporter;
+
 import org.broadinstitute.barclay.argparser.Hidden;
+import org.broadinstitute.barclay.help.scanners.JavaLanguageModelScanners;
 import org.broadinstitute.barclay.utils.JVMUtils;
 
+import javax.lang.model.SourceVersion;
+import javax.lang.model.element.Element;
 import java.io.*;
 import java.util.*;
 
@@ -43,37 +46,19 @@ import java.util.*;
  * want to subclass it to override the following methods in order to create application-specific
  * templates and template property maps:
  *
- * {@link #getIndexTemplateName}
- * {@link #createWorkUnit}
- * {@link #createGSONWorkUnit}
- * {@link #start(RootDoc)} A static method that instantiates the subclass and delegates to the
- * instance method {@link #startProcessDocs(RootDoc)}.
+ * {@link #run(DocletEnvironment)}
+ * {@link #getIndexTemplateName()}
+ * {@link #createWorkUnit(Element, Class, DocumentedFeature)}
+ * {@link #createGSONWorkUnit(DocWorkUnit, List, List)}
  */
-public class HelpDoclet {
-    final protected static Logger logger = LogManager.getLogger(HelpDoclet.class);
-
-    // Builtin javadoc command line arguments
-    final private static String DESTINATION_DIR_OPTION = "-d";
-    final private static String WINDOW_TITLE_OPTION = "-windowtitle";
-    final private static String DOC_TITLE_OPTION = "-doctitle";
-    final private static String QUIET_OPTION = "-quiet";
-
-    // Barclay HelpDoclet custom command line options
-    final private static String SETTINGS_DIR_OPTION = "-settings-dir";
-    final private static String BUILD_TIMESTAMP_OPTION = "-build-timestamp";
-    final private static String ABSOLUTE_VERSION_OPTION = "-absolute-version";
-    final private static String INCLUDE_HIDDEN_OPTION = "-hidden-version";
-    final private static String OUTPUT_FILE_EXTENSION_OPTION = "-output-file-extension";
-    final private static String INDEX_FILE_EXTENSION_OPTION = "-index-file-extension";
-    final private static String USE_DEFAULT_TEMPLATES_OPTION = "-use-default-templates";
-
+public class HelpDoclet implements Doclet {
     // Where we find the help FreeMarker templates
-    final private static File DEFAULT_SETTINGS_DIR = new File("settings/helpTemplates");
-    final private static String DEFAULT_SETTINGS_CLASSPATH = "/org/broadinstitute/barclay/helpTemplates";
+    final static File DEFAULT_SETTINGS_DIR = new File("settings/helpTemplates");
+    final static String DEFAULT_SETTINGS_CLASSPATH = "/org/broadinstitute/barclay/helpTemplates";
     // Where we write the output
-    final private static File DEFAULT_DESTINATION_DIR = new File("barclaydocs");
+    final static File DEFAULT_DESTINATION_DIR = new File("barclaydocs");
     // Default output file extension
-    final private static String DEFAULT_OUTPUT_FILE_EXTENSION = "html";
+    final static String DEFAULT_OUTPUT_FILE_EXTENSION = "html";
 
     // ----------------------------------------------------------------------
     //
@@ -91,20 +76,181 @@ public class HelpDoclet {
     protected boolean useDefaultTemplates = false;
 
     // Variables to store data for Freemarker:
-    private RootDoc rootDoc;                // The javadoc root doc
-    protected Set<DocWorkUnit> workUnits;     // Set of all things we are going to document
+    private DocletEnvironment docletEnv;      // The javadoc doclet env
+    protected Set<DocWorkUnit> workUnits = new LinkedHashSet<>();     // Set of all things we are going to document
+    private  Reporter reporter;
 
-    /**
-     * The entry point for javadoc generation. Default implementation creates an instance of
-     * {@link HelpDoclet} and calls {@link #startProcessDocs(RootDoc)} on that instance.
-     *
-     * <p>Note: Custom Barclay doclets should subclass this class, and implement a similar static
-     * method that creates an instance of the doclet subclass and delegates to that instance's
-     * {@link #startProcessDocs(RootDoc)}.
-     */
-     public static boolean start(final RootDoc rootDoc) throws IOException {
-         return new HelpDoclet().startProcessDocs(rootDoc);
-     }
+    @Override
+    public void init(final Locale locale, final Reporter reporter) {
+        this.reporter = reporter;
+    }
+
+    @Override
+    public String getName() {
+        return getClass().getName();
+    }
+
+    @Override
+    public boolean run(final DocletEnvironment docEnv) {
+        // Make sure the user specified a settings directory OR that we should use the defaults.
+        // Both are not allowed.
+        // Neither are not allowed.
+        if ( (useDefaultTemplates && isSettingsDirSet) ||
+                (!useDefaultTemplates && !isSettingsDirSet)) {
+            throw new RuntimeException("ERROR: must specify only ONE of: " + BarclayDocletOptions.USE_DEFAULT_TEMPLATES_OPTION + " , " + BarclayDocletOptions.SETTINGS_DIR_OPTION);
+        }
+
+        this.docletEnv = docEnv;
+
+        // Make sure we can use the directory for settings we have set:
+        if (!useDefaultTemplates) {
+            validateSettingsDir();
+        }
+
+        // Make sure we're in a good state to run:
+        validateDocletStartingState();
+
+        processDocs(docEnv);
+        return true;
+    }
+
+    @Override
+    public Set<? extends Option> getSupportedOptions() {
+        return new LinkedHashSet<>() {{
+
+            // standard javadoc options
+
+            add(new BarclayDocletOption.SimpleStandardOption(BarclayDocletOptions.DESTINATION_DIR_OPTION) {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    destinationDir = new File(arguments.get(0));
+                    return true;
+                }
+            });
+
+            add(new BarclayDocletOption.SimpleStandardOption(BarclayDocletOptions.WINDOW_TITLE_OPTION) {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    return true;
+                }
+            });
+
+            add(new BarclayDocletOption.SimpleStandardOption(BarclayDocletOptions.DOC_TITLE_OPTION) {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    return true;
+                }
+            });
+
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.QUIET_OPTION),
+                    "quiet",
+                    0,
+                    Option.Kind.STANDARD,
+                    null) {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    // the gradle javadoc task includes this since its a standard doclet arg
+                    // so we need to tolerate it, but ignore it
+                    return true;
+                }
+            });
+
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.SETTINGS_DIR_OPTION),
+                    "settings dir",
+                    1,
+                    Option.Kind.STANDARD,
+                    "<string>") {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    settingsDir = new File(arguments.get(0));
+                    isSettingsDirSet = true;
+                    return true;
+                }
+            });
+
+            // custom Barclay options
+
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.BUILD_TIMESTAMP_OPTION),
+                    "build timestamp",
+                    1,
+                    Option.Kind.OTHER,
+                    "<string>") {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    buildTimestamp = arguments.get(0);
+                    return true;
+                }
+            });
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.ABSOLUTE_VERSION_OPTION),
+                    "absolute version",
+                    1,
+                    Option.Kind.OTHER,
+                    "<string>") {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    absoluteVersion = arguments.get(0);
+                    return true;
+                }
+            });
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.OUTPUT_FILE_EXTENSION_OPTION),
+                    "output file extension",
+                    1,
+                    Option.Kind.OTHER,
+                    "<string>") {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    outputFileExtension = arguments.get(0);
+                    return true;
+                }
+            });
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.INDEX_FILE_EXTENSION_OPTION),
+                    "index file extension",
+                    1,
+                    Option.Kind.OTHER,
+                    "<string>") {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    indexFileExtension = arguments.get(0);
+                    return true;
+                }
+            });
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.INCLUDE_HIDDEN_OPTION),
+                    "hidden features",
+                    0,
+                    Option.Kind.OTHER,
+                    null) {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    showHiddenFeatures = true;
+                    return true;
+                }
+            });
+            add(new BarclayDocletOption(
+                    Arrays.asList(BarclayDocletOptions.USE_DEFAULT_TEMPLATES_OPTION),
+                    "use default templates",
+                    0,
+                    Option.Kind.OTHER,
+                    null) {
+                @Override
+                public boolean process(String option, List<String> arguments) {
+                    useDefaultTemplates = true;
+                    return true;
+                }
+            });
+        }};
+    }
+
+    @Override
+    public SourceVersion getSupportedSourceVersion() {
+        return SourceVersion.RELEASE_17;
+    }
 
     /**
      * Ensure that {@link #settingsDir} exists and is a directory.
@@ -112,10 +258,10 @@ public class HelpDoclet {
      */
     private void validateSettingsDir() {
          if (!settingsDir.exists()) {
-             throw new RuntimeException(SETTINGS_DIR_OPTION + " : " + settingsDir.getPath() + " does not exist!");
+             throw new RuntimeException(BarclayDocletOptions.SETTINGS_DIR_OPTION + " : " + settingsDir.getPath() + " does not exist!");
          }
          else if (!settingsDir.isDirectory()) {
-             throw new RuntimeException(SETTINGS_DIR_OPTION + " : " + settingsDir.getPath() + " is not a directory!");
+             throw new RuntimeException(BarclayDocletOptions.SETTINGS_DIR_OPTION + " : " + settingsDir.getPath() + " is not a directory!");
          }
      }
 
@@ -130,136 +276,17 @@ public class HelpDoclet {
     }
 
     /**
-     * Extracts the contents of certain types of javadoc and adds them to an output file.
+     * Process the classes that have been included by the javadoc process.
      *
-     * @param rootDoc The documentation root.
-     * @return Whether the JavaDoc run succeeded.
-     * @throws java.io.IOException if output can't be written.
+     * @param docletEnv the Doclet's DocletEnvironment
      */
-    protected boolean startProcessDocs(final RootDoc rootDoc) throws IOException {
-        for (String[] options : rootDoc.options()) {
-            parseOption(options);
-        }
+    private void processDocs(final DocletEnvironment docletEnv) {
+        this.docletEnv = docletEnv;
 
-        // Make sure the user specified a settings directory OR that we should use the defaults.
-        // Both are not allowed.
-        // Neither are not allowed.
-        if ( (useDefaultTemplates && isSettingsDirSet) ||
-                (!useDefaultTemplates && !isSettingsDirSet)) {
-            throw new RuntimeException("ERROR: must specify only ONE of: " + USE_DEFAULT_TEMPLATES_OPTION + " , " + SETTINGS_DIR_OPTION);
-        }
+        // scan all included elements for DocumentedFeatures
+        workUnits = JavaLanguageModelScanners.getWorkUnits(this, docletEnv, reporter, docletEnv.getIncludedElements());
 
-        // Make sure we can use the directory for settings we have set:
-        if (!useDefaultTemplates) {
-            validateSettingsDir();
-        }
-
-        // Make sure we're in a good state to run:
-        validateDocletStartingState();
-
-
-
-        processDocs(rootDoc);
-        return true;
-    }
-
-    /**
-     * Handles javadoc command line options. The first entry in the {@code options} array is the
-     * option name. Subsequent entries contain the option's arguments, the number of which is
-     * determined by the value returned from {@link #optionLength(String)} for that option.
-     *
-     * <p>Custom Barclay doclets that want to have custom command line options should override this
-     * method, and also provide an implementation of the static method {@link #optionLength}).
-     * Both methods should handle the custom options, and then delegate back to the base class
-     * methods defined here to allow default handling of builtin options.
-     *
-     * @param options Options to parse.
-     * @return True if {@code options} was parsed, False otherwise.
-     */
-    protected boolean parseOption(final String[] options) {
-
-        boolean hasParsedOption = false;
-
-        if (options[0].equals(SETTINGS_DIR_OPTION)) {
-            settingsDir = new File(options[1]);
-            isSettingsDirSet = true;
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(DESTINATION_DIR_OPTION)) {
-            destinationDir = new File(options[1]);
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(BUILD_TIMESTAMP_OPTION)) {
-            buildTimestamp = options[1];
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(ABSOLUTE_VERSION_OPTION)) {
-            absoluteVersion = options[1];
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(INCLUDE_HIDDEN_OPTION)) {
-            showHiddenFeatures = true;
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(OUTPUT_FILE_EXTENSION_OPTION)) {
-            outputFileExtension = options[1];
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(INDEX_FILE_EXTENSION_OPTION)) {
-            indexFileExtension = options[1];
-            hasParsedOption = true;
-        }
-        else if (options[0].equals(USE_DEFAULT_TEMPLATES_OPTION)) {
-            useDefaultTemplates = true;
-            hasParsedOption = true;
-        }
-
-        return hasParsedOption;
-    }
-    /**
-     * Validates the given options against options supported by this doclet.
-     *
-     * <p>Custom Barclay doclets that want to have custom command line options should provide an
-     * implementation of this static method, and also override override {@link #parseOption}).
-     * Both methods should handle the custom options, delegating back to the base class methods
-     * defined here to allow default handling of builtin options.
-     *
-     * @param option Option to validate.
-     * @return Number of potential parameters; 0 if not supported.
-     */
-    public static int optionLength(final String option) {
-        // Any arguments used for the doclet need to be recognized here. Many javadoc plugins (ie. gradle)
-        // automatically add some such as "-doctitle", "-windowtitle", which we ignore.
-        if (option.equals(DOC_TITLE_OPTION) ||
-            option.equals(WINDOW_TITLE_OPTION) ||
-            option.equals(SETTINGS_DIR_OPTION) ||
-            option.equals(DESTINATION_DIR_OPTION) ||
-            option.equals(BUILD_TIMESTAMP_OPTION) ||
-            option.equals(ABSOLUTE_VERSION_OPTION) ||
-            option.equals(OUTPUT_FILE_EXTENSION_OPTION) ||
-            option.equals(INDEX_FILE_EXTENSION_OPTION)) {
-            return 2;
-        } else if (option.equals(QUIET_OPTION) ||
-                   option.equals(USE_DEFAULT_TEMPLATES_OPTION)) {
-            return 1;
-        } else {
-            logger.error("The Javadoc command line option is not recognized by the Barclay doclet: " + option);
-            return 0;
-        }
-    }
-
-    /**
-     * Process the classes that have been included by the javadoc process in the rootDoc object.
-     *
-     * @param rootDoc root structure containing the the set of objects accumulated by the javadoc process
-     */
-    private void processDocs(final RootDoc rootDoc) {
-        this.rootDoc = rootDoc;
-
-        // Get a list of all the features and groups that we'll actually retain
-        workUnits = computeWorkUnits();
-
-        final Set<String> uniqueGroups = new HashSet<>();
+        final Set<String> uniqueGroups = new LinkedHashSet<>();
         final List<Map<String, String>> featureMaps = new ArrayList<>();
         final List<Map<String, String>> groupMaps = new ArrayList<>();
 
@@ -274,46 +301,14 @@ public class HelpDoclet {
                 }
         );
 
-        // Second pass:  populate the property map for each work unit
+        // Second pass: populate the property map for each work unit
         workUnits.stream().forEach(workUnit -> { workUnit.processDoc(featureMaps, groupMaps); });
 
         // Third pass: Generate the individual outputs for each work unit, and the top-level index file
         emitOutputFromTemplates(groupMaps, featureMaps);
     }
 
-
-    /**
-     * For each class in the rootDoc class list, delegate to the appropriate DocWorkUnitHandler to
-     * determine if it should be included in this run, and for each included feature, construct a DocWorkUnit.
-     *
-     * @return the set of all DocWorkUnits for which we are actually generating docs
-     */
-    private Set<DocWorkUnit> computeWorkUnits() {
-        final TreeSet<DocWorkUnit> workUnits = new TreeSet<>();
-
-        for (final ClassDoc classDoc : rootDoc.classes()) {
-            final Class<?> clazz = getClassForClassDoc(classDoc);
-            final DocumentedFeature documentedFeature = getDocumentedFeatureForClass(clazz);
-
-            if (documentedFeature != null) {
-                if (documentedFeature.enable() && includeInDocs(documentedFeature, classDoc, clazz)) {
-                        DocWorkUnit workUnit = createWorkUnit(
-                            documentedFeature,
-                            classDoc,
-                            clazz);
-                    if (workUnit != null) {
-                        workUnits.add(workUnit);
-                    }
-                } else {
-                    logger.info("Skipping disabled documentation for feature: " + classDoc);
-                }
-            }
-        }
-
-        return workUnits;
-    }
-
-    public RootDoc getRootDoc() { return rootDoc; }
+    public DocletEnvironment getDocletEnv() { return docletEnv; }
 
     public String getBuildTimeStamp() { return buildTimestamp; }
 
@@ -354,11 +349,10 @@ public class HelpDoclet {
      * to determine if a DocWorkUnit should be created for this feature.
      *
      * @param documentedFeature feature that is being considered for inclusion in the docs
-     * @param classDoc for the class that is being considered for inclusion in the docs
      * @param clazz class that is being considered for inclusion in the docs
      * @return true if the doc should be included, otherwise false
      */
-    public boolean includeInDocs(final DocumentedFeature documentedFeature, final ClassDoc classDoc, final Class<?> clazz) {
+    public boolean includeInDocs(final DocumentedFeature documentedFeature, final Class<?> clazz) {
         boolean hidden = !showHiddenFeatures() && clazz.isAnnotationPresent(Hidden.class);
         return !hidden && JVMUtils.isConcrete(clazz);
     }
@@ -407,50 +401,17 @@ public class HelpDoclet {
      * Create a work unit and handler capable of documenting the feature specified by the input arguments.
      * Returns null if no appropriate handler is found or doc shouldn't be documented at all.
      */
-    protected DocWorkUnit createWorkUnit(
-            final DocumentedFeature documentedFeature,
-            final ClassDoc classDoc,
-            final Class<?> clazz)
+    public DocWorkUnit createWorkUnit(
+            final Element classElement,
+            final Class<?> clazz,
+            final DocumentedFeature documentedFeature)
     {
         return new DocWorkUnit(
                 new DefaultDocWorkUnitHandler(this),
-                documentedFeature,
-                classDoc,
-                clazz);
-    }
-
-    /**
-     * Returns the instantiated DocumentedFeature that describes the doc for this class.
-     *
-     * @param clazz
-     * @return DocumentedFeature, or null if this classDoc shouldn't be included/documented
-     */
-    private DocumentedFeature getDocumentedFeatureForClass(final Class<?> clazz) {
-        if (clazz != null && clazz.isAnnotationPresent(DocumentedFeature.class)) {
-            return clazz.getAnnotation(DocumentedFeature.class);
-        }
-        else {
-            return null;
-        }
-    }
-
-    /**
-     * Return the Java class described by the ClassDoc doc
-     *
-     * @param doc
-     * @return
-     */
-    private Class<? extends Object> getClassForClassDoc(final ClassDoc doc) {
-        try {
-            return DocletUtils.getClassForDoc(doc);
-        } catch (ClassNotFoundException e) {
-            // we got a classdoc for a class we can't find.  Maybe in a library or something
-            return null;
-        } catch (NoClassDefFoundError e) {
-            return null;
-        } catch (UnsatisfiedLinkError e) {
-            return null; // naughty BWA bindings
-        }
+                classElement,
+                clazz,
+                documentedFeature
+        );
     }
 
     /**
@@ -562,16 +523,6 @@ public class HelpDoclet {
         return null;
     }
 
-    /**
-     * Return the ClassDoc associated with clazz
-     *
-     * @param clazz
-     * @return
-     */
-    public ClassDoc getClassDocForClass(final Class<?> clazz) {
-        return rootDoc.classNamed(clazz.getName());
-    }
-
     /**
      * High-level function that processes a single DocWorkUnit unit using its handler
      *
