@@ -1,8 +1,8 @@
 package org.broadinstitute.barclay.argparser;
 
 import org.apache.commons.lang3.StringUtils;
-import org.apache.commons.lang3.tuple.Pair;
 
+import org.broadinstitute.barclay.utils.Pair;
 import org.testng.Assert;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
@@ -110,7 +110,7 @@ public final class CommandLineArgumentParserTest {
                     new FrobnicateArguments(), Arrays.asList(
                         "--FROBNICATION_FLAVOR <FrobnicationFlavor>",
                         "--SHMIGGLE_TYPE <String>",
-                        "--arguments_file <File>",
+                        "--arguments-file,-arguments_file <File>",
                         "--FROBNICATION_THRESHOLD,-T <Integer>",
                         "--TRUTHINESS <Boolean>")
                 },
@@ -352,7 +352,7 @@ public final class CommandLineArgumentParserTest {
                 "FrobnicateArguments " +
                         "positional1 positional2 --FROBNICATION_THRESHOLD 17 --FROBNICATION_FLAVOR BAR " +
                         "--SHMIGGLE_TYPE shmiggle1 --SHMIGGLE_TYPE shmiggle2 --TRUTHINESS true --help false " +
-                        "--version false --showHidden false");
+                        "--version false --show-hidden false");
     }
 
     private static class WithSensitiveValues {
